package ru.amse.vorobiev.lce.circuit;

import java.util.HashSet;
import java.util.List;
import java.util.LinkedList;
import java.util.Set;

import ru.amse.vorobiev.lce.elements.*;
import ru.amse.vorobiev.lce.elements.impl.GateType;
import ru.amse.vorobiev.lce.elements.impl.InGate;
import ru.amse.vorobiev.lce.elements.impl.OutGate;
import ru.amse.vorobiev.lce.interpreter.IInterpreter;
import ru.amse.vorobiev.lce.interpreter.Interpreter;
import ru.amse.vorobiev.lce.ui.IChangeListener;

public class Circuit implements ICircuit {
	private Set<IGate> myGates = new HashSet<IGate>();
	private Set<InGate> myInGates = new HashSet<InGate>();
	private Set<OutGate> myOutGates = new HashSet<OutGate>();
	private List<IChangeListener> myChangeListeners = new LinkedList<IChangeListener>();
	private final IInterpreter myInterpreter = new Interpreter(this);

	public void addGate(IGate gate) {
		if (gate != null) {
			gate.setCircuit(this);
			if (gate.getType() == GateType.IN) {
				myInGates.add((InGate) gate);
			}
			if (gate.getType() == GateType.OUT) {
				myOutGates.add((OutGate) gate);
			}
			myGates.add(gate);
		}
	}

	public void addSubcircuit(InGate gate) {
		LinkedList<IGate> queue = new LinkedList<IGate>();
		Set<IGate> visited = new HashSet<IGate>();
		queue.add(gate);
		gate.setCircuit(this);
		myInGates.add(gate);
		IGate nextGate;
		while (!queue.isEmpty()) {
			nextGate = queue.poll();
			visited.add(nextGate);
			if (nextGate.getType() != GateType.OUT) {
				for (int j = 0; j < nextGate.getOutputCount(); j++) {
					IPlug output = nextGate.getOutput(j);
					if (output.getWire() != null) {
						IPlug input = output.getWire().getTo();
						IGate gateToAdd = input.getGate();
						if (!visited.contains(gateToAdd) && queue.indexOf(gateToAdd) == -1) {
							queue.add(gateToAdd);
						}
					}
				}
			} else {
				myOutGates.add((OutGate) nextGate);
			}
			myGates.add(nextGate);
			nextGate.setCircuit(this);
		}
	}
	
	public IInterpreter getInterpreter() {
		return myInterpreter;
	}

	public Set<InGate> getInGates() {
		return myInGates;
	}
	
	public Set<OutGate> getOutGates() {
		return myOutGates;
	}
	
	public Set<IGate> getAllGates() {
		return myGates;
	}

	public int size() {
		return myGates.size();
	}

	public void addChangeListener(IChangeListener listener) {
		myChangeListeners.add(listener);
	}

	public void fireCircuitChanged() {
		for (IChangeListener listener : myChangeListeners) {
			listener.changed();
		}
	}

	public List<IChangeListener> getChangeListeners() {
		return myChangeListeners;
	}

	public void removeGate(IGate gate) {
		for (int i = 0; i < gate.getInputCount(); i++) {
			IWire wire = gate.getInput(i).getWire();
			if (wire != null) {
				wire.getTo().setWire(null);
				wire.setTo(null);
				wire.getFrom().setWire(null);
				wire.setFrom(null);
			}
		}
		for (int i = 0; i < gate.getOutputCount(); i++) {
			IWire wire = gate.getOutput(i).getWire();
			if (wire != null) {
				wire.getTo().setWire(null);
				wire.setTo(null);
				wire.getFrom().setWire(null);
				wire.setFrom(null);
			}
		}
		myGates.remove(gate);
		myInGates.remove(gate);
		myOutGates.remove(gate);
		fireCircuitChanged();
	}

}
