package ru.amse.vorobiev.lce.circuit;

import java.util.List;

import ru.amse.vorobiev.lce.interpreter.IInterpreter;
import ru.amse.vorobiev.lce.ui.ErrorTableModel;
import ru.amse.vorobiev.lce.ui.IChangeListener;
import ru.amse.vorobiev.lce.ui.JEditorFrame;
import ru.amse.vorobiev.lce.validator.IError;
import ru.amse.vorobiev.lce.validator.IValidator;
import ru.amse.vorobiev.lce.validator.Validator;

public class CircuitChangeListener implements IChangeListener{
	private final JEditorFrame myFrame;
	private final ICircuit myCircuit;
	private final IValidator myValidator;
	
	public CircuitChangeListener(ICircuit circuit, JEditorFrame frame) {
		myCircuit = circuit;
		myFrame = frame;
		myValidator = new Validator();
	}

	public void changed() {
		List<IError> errors = myValidator.validate(myCircuit);
		((ErrorTableModel) myFrame.getErrorTable().getModel()).setErrors(errors);
		IInterpreter interpreter = myCircuit.getInterpreter();
		interpreter.setUp(myCircuit);
		interpreter.reset();
		if (errors.isEmpty() && interpreter.allInGatesAreReady()) {
			myFrame.setInterpreter(interpreter);
			myFrame.setInterpreterActive(true);
		} else {
			myFrame.setInterpreterActive(false);
		}
		myFrame.getEditor().fireEditorChanged();
	}
	
	
}
