package ru.amse.vorobiev.lce.io;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import ru.amse.vorobiev.lce.circuit.ICircuit;
import ru.amse.vorobiev.lce.elements.IGate;
import ru.amse.vorobiev.lce.elements.IWire;
import ru.amse.vorobiev.lce.view.ICircuitView;
import ru.amse.vorobiev.lce.view.IGateView;

public class CircuitSaver {
	
	public void saveCircuitToFile(ICircuitView circuitView, File file) throws FileNotFoundException, XMLStreamException {
		if (circuitView != null) {
			ICircuit circuit = circuitView.getCircuit();
			Map<IGate, Integer> gateToIdMap = new HashMap<IGate, Integer>();
			int currentId = 0;
			OutputStream stream = new FileOutputStream(file);
			XMLOutputFactory f = XMLOutputFactory.newInstance();
			XMLStreamWriter w = f.createXMLStreamWriter(stream);
			try {
				w.writeStartDocument();
				w.writeCharacters("\n");
				w.writeStartElement("circuit");
				w.writeStartElement("model");
				w.writeStartElement("gates");
				for (IGate gate : circuit.getAllGates()) {
					gateToIdMap.put(gate, currentId);
					w.writeStartElement("gate");
					w.writeAttribute("id", currentId + "");
					w.writeAttribute("type", gate.getType() + "");
					w.writeAttribute("inputs", gate.getInputCount() + "");
					w.writeAttribute("outputs", gate.getOutputCount() + "");
					w.writeEndElement();
					w.writeCharacters("\n");
					currentId++;
				}
				w.writeEndElement();
				w.writeStartElement("wires");
				for (IGate gate : circuit.getAllGates()) {
					for (int j = 0; j < gate.getInputCount(); j++) {
						IWire wire = gate.getInput(j).getWire();
						if (wire != null) {
							int startId = gateToIdMap.get(wire.getFrom().getGate());
							int endId = gateToIdMap.get(wire.getTo().getGate());
							int startIndex = wire.getFrom().getGate().getOutputIndex(wire.getFrom());
							w.writeStartElement("wire");
							w.writeAttribute("from", startId + "");
							w.writeAttribute("output", startIndex + "");
							w.writeAttribute("to", endId + "");
							w.writeAttribute("input", j + "");
							w.writeEndElement();
							w.writeCharacters("\n");
						}
					}
				}
				w.writeEndElement();
				w.writeEndElement();
				w.writeStartElement("view");
				for (IGate gate : circuit.getAllGates()) {
					IGateView gateView = (IGateView) circuitView.getViewByElement(gate);
					int id = gateToIdMap.get(gate);
					w.writeStartElement("gateview");
					w.writeAttribute("id", id + "");
					w.writeAttribute("x", gateView.getUpperLeftCorner().x + "");
					w.writeAttribute("y", gateView.getUpperLeftCorner().y + "");
					w.writeEndElement();
				}
				w.writeEndElement();
				w.writeEndDocument();	      
			} finally {
				w.close();
			}
		}
	}
}
