package ru.amse.vorobiev.lce.ui;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.swing.JComponent;

import ru.amse.vorobiev.lce.circuit.Circuit;
import ru.amse.vorobiev.lce.elements.IGate;
import ru.amse.vorobiev.lce.elements.impl.InGate;
import ru.amse.vorobiev.lce.interpreter.IInterpreter;
import ru.amse.vorobiev.lce.ui.tools.ITool;
import ru.amse.vorobiev.lce.view.ICircuitView;
import ru.amse.vorobiev.lce.view.IGateView;
import ru.amse.vorobiev.lce.view.impl.CircuitView;

public class JEditor extends JComponent {
	public static final int PLUG_DIAMETER = 4;
	public static final int MIN_SPACE_BEETWEEN_PLUGS = 10;
	public static final int GATE_TO_PLUG_WIRE_LENGTH = 5;
	public static final int GATE_INITIAL_HEIGHT = 40;
	public static final int GATE_INITIAL_WIDTH = 45;
	public static final int PLUG_CONTROL_LENGTH = 8;
	public static final int PLUG_DELETING_TOOL_HEIGHT = 4;
	public static final int PLUG_DELETING_TOOL_WIDTH = 6;

	private static final long serialVersionUID = 1L;
	private ICircuitView myView;
	private final Set<IGateView> mySelectedGates;
	private final Set<ITool> myCurrentTools;
	private List<IChangeListener> myChangeListeners = new LinkedList<IChangeListener>();

	public JEditor() {
		myView = new CircuitView(new Circuit());
		myCurrentTools = new HashSet<ITool>();
		mySelectedGates = new HashSet<IGateView>();
	}
	
	public Set<ITool> getCurrentTools() {
		return myCurrentTools;
	}
	
	private void clearCurrentTools() {
		for (ITool tool : myCurrentTools) {
			this.removeMouseListener(tool);
			this.removeMouseMotionListener(tool);
		}
		myCurrentTools.clear();
	}
	
	public void setCurrentTool(ITool tool) {
		clearCurrentTools();
		myCurrentTools.add(tool);
		this.addMouseListener(tool);
		this.addMouseMotionListener(tool);
	}
	
	public void setCurrentTools(Set<ITool> tools) {
		clearCurrentTools();
		for (ITool tool : tools) {
			myCurrentTools.add(tool);
			this.addMouseListener(tool);
			this.addMouseMotionListener(tool);
		}
	}
	
	public Set<IGateView> getSelectedGates() {
		return mySelectedGates;
	}
	
	public void clearSelection() {
		mySelectedGates.clear();
	}
	
	public void setView(ICircuitView view) {
		clearSelection();
		myView = view;
	}
	
	public ICircuitView getView(){
		return myView;
	}
	
	public void ensureSize() {
		Dimension dimension = new Dimension(0, 0);
		for (IGate gate : myView.getCircuit().getAllGates()) {
			IGateView gateView = (IGateView) myView.getViewByElement(gate);
			Point point = gateView.getUpperLeftCorner();
			if ((point.x + gateView.getWidth()) > dimension.width) {
				dimension.width = point.x + gateView.getWidth() + 10;
			}
			if ((point.y + gateView.getHeight()) > dimension.height) {
				dimension.height = point.y + gateView.getHeight() + 10;
			}
		}
		this.setSize(dimension);
		this.setPreferredSize(dimension);
	}
	
	public boolean setInputs(String inputString, IGateView gateView) {
		IGate gate = gateView.getGate();
		int inputCount = gate.getOutputCount();
		boolean isCorrectString = true;
		boolean [] inputs = new boolean[inputCount];
		if (inputString.length() >= inputCount) {
			for(int j = 0; j < gate.getOutputCount(); j++) {
				char inputChar = inputString.charAt(j);
				if (inputChar != '0' && inputChar != '1') {
					isCorrectString = false;
				}
				inputs[j] = (inputChar == '0') ? false : true; 
			}
		} else {
			isCorrectString = false;
		}
		if (isCorrectString) {
			IInterpreter interpreter = getView().getCircuit().getInterpreter();
			interpreter.setInputs((InGate) gate, inputs);
			myView.getCircuit().fireCircuitChanged();
			repaint();
			return true;
		} else {
			return false;
		}
	}
	
	public void addChangeListener(IChangeListener listener) {
		myChangeListeners.add(listener);
	}

	public void fireEditorChanged() {
		for (IChangeListener listener : myChangeListeners) {
			listener.changed();
		}
	}

	public List<IChangeListener> getChangeListeners() {
		return myChangeListeners;
	}
	
	public void paint(Graphics g) {	
		myView.paint(g);
		for (ITool tool : myCurrentTools) {
			tool.paint(g);
		}
	}
}
