package ru.amse.vorobiev.lce.ui.tools.impl;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Stroke;
import java.awt.event.MouseEvent;

import ru.amse.vorobiev.lce.ui.JEditor;
import ru.amse.vorobiev.lce.ui.tools.ITool;
import ru.amse.vorobiev.lce.view.ICircuitView;

public abstract class AbstractGateCreationTool implements ITool {
	private Point myMousePosition;
	private JEditor myEditor;
	private boolean needToDraw;
	
	public AbstractGateCreationTool (JEditor editor) {
		myEditor = editor;
		myMousePosition = new Point(0, 0);
		needToDraw = false;
	}

	public void paint(Graphics g) {
		if (needToDraw) {
			Graphics2D g2 = (Graphics2D) g;
		
			g2.setColor(Color.GRAY);
			Stroke thindashed = new BasicStroke(1.0f, BasicStroke.CAP_BUTT, 
											BasicStroke.JOIN_BEVEL, 1.0f, 
											new float[] { 8.0f, 3.0f}, 0.0f);
			g2.setStroke(thindashed);
			g2.drawRect(myMousePosition.x, myMousePosition.y, JEditor.GATE_INITIAL_WIDTH, JEditor.GATE_INITIAL_HEIGHT);
		}
	}

	public void mouseClicked(MouseEvent e) {
		createNewGate(e.getX(), e.getY(), myEditor.getView());
		myEditor.getView().getCircuit().fireCircuitChanged();
		myEditor.ensureSize();
		myEditor.repaint();
	}

	public void mouseEntered(MouseEvent e) {
		needToDraw = true;
	}

	public void mouseExited(MouseEvent e) {
		needToDraw = false;
		myEditor.repaint();
	}

	public void mousePressed(MouseEvent e) {
	}

	public void mouseReleased(MouseEvent e) {
	}

	public void mouseDragged(MouseEvent e) {
		myMousePosition.x = e.getX();
		myMousePosition.y = e.getY();
		myEditor.repaint();
	}

	public void mouseMoved(MouseEvent e) {
		myMousePosition.x = e.getX();
		myMousePosition.y = e.getY();
		myEditor.repaint();
	}
	
	protected abstract void createNewGate(int x, int y, ICircuitView view);
}
