package ru.amse.vorobiev.lce.ui.tools.impl;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Stroke;
import java.awt.event.MouseEvent;
import java.util.Set;

import ru.amse.vorobiev.lce.elements.IGate;
import ru.amse.vorobiev.lce.ui.JEditor;
import ru.amse.vorobiev.lce.ui.tools.ITool;
import ru.amse.vorobiev.lce.view.IGateView;

public class GateSelectionTool implements ITool {
	private JEditor myEditor;
	private Point mySelectionStartPoint;
	private Point myMousePosition;
	
	public GateSelectionTool(JEditor editor) {
		myEditor = editor;
		mySelectionStartPoint = null;
		myMousePosition = new Point();
	}
	
	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		Set<IGateView> selectedGates = myEditor.getSelectedGates();
		if (!selectedGates.isEmpty()) {
			g2.setColor(Color.BLUE);
			for (IGateView selectedGate : selectedGates) {
				g2.drawRect(selectedGate.getUpperLeftCorner().x - 3, 
						selectedGate.getUpperLeftCorner().y - 3,
						selectedGate.getWidth() + 6, 
						selectedGate.getHeight() + 6);
			}
		}
		if (mySelectionStartPoint != null) {
			g2.setColor(Color.GRAY);
			Stroke thindashed = new BasicStroke(1.0f, BasicStroke.CAP_BUTT, 
												BasicStroke.JOIN_BEVEL, 1.0f, 
												new float[] { 8.0f, 3.0f}, 0.0f);
			g2.setStroke(thindashed);
			int startX = (mySelectionStartPoint.x < myMousePosition.x)
							? mySelectionStartPoint.x
							: myMousePosition.x;
			int startY = (mySelectionStartPoint.y < myMousePosition.y)
							? mySelectionStartPoint.y
							: myMousePosition.y;
			g2.drawRect(startX, startY, 
					Math.abs(myMousePosition.x - mySelectionStartPoint.x),
					Math.abs(myMousePosition.y - mySelectionStartPoint.y));
		}
	}
	
	public void mouseClicked(MouseEvent e) {
		int x = e.getX();
		int y = e.getY();
		IGateView selectedGate = myEditor.getView().getGateForLocation(x, y);
		if (selectedGate != null) {
			myEditor.clearSelection();
			myEditor.getSelectedGates().add(selectedGate);
			myEditor.repaint();
		}
	}

	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void mousePressed(MouseEvent e) {
		int x = e.getX();
		int y = e.getY();
		IGateView selectedGate = myEditor.getView().getGateForLocation(x, y);
		
		if (selectedGate != null && !myEditor.getSelectedGates().contains(selectedGate)) {
			myEditor.clearSelection();
			myEditor.getSelectedGates().add(selectedGate);
		} else if (selectedGate == null && myEditor.getView().getPlugForLocation(x, y) == null) {
			myEditor.clearSelection();
			mySelectionStartPoint = new Point(x, y);
		}
		myEditor.repaint();
	}

	public void mouseReleased(MouseEvent arg0) {
		mySelectionStartPoint = null;
		myEditor.repaint();
	}

	public void mouseDragged(MouseEvent e) {
		myMousePosition.x = e.getX();
		myMousePosition.y = e.getY();
		
		if (mySelectionStartPoint != null) {
			int startX = (mySelectionStartPoint.x < myMousePosition.x)
					? mySelectionStartPoint.x
					: myMousePosition.x;	
			int startY = (mySelectionStartPoint.y < myMousePosition.y)
					? mySelectionStartPoint.y
					: myMousePosition.y;
			int width = Math.abs(myMousePosition.x - mySelectionStartPoint.x);
			int height = Math.abs(myMousePosition.y - mySelectionStartPoint.y);
		
			myEditor.clearSelection();
			for (IGate gate : myEditor.getView().getCircuit().getAllGates()) {
				IGateView gateView = (IGateView) myEditor.getView().getViewByElement(gate);
				int x = gateView.getUpperLeftCorner().x;
				int y = gateView.getUpperLeftCorner().y;
				if (x > startX && y > startY && x < startX + width && y < startY + height ||
						x + gateView.getWidth() > startX && y > startY && x + gateView.getWidth() < startX + width && y < startY + height ||
						x > startX && y + gateView.getHeight() > startY && x < startX + width && y + gateView.getHeight() < startY + height ||
						x + gateView.getWidth() > startX && y + gateView.getHeight() > startY && x + gateView.getWidth() < startX + width && y + gateView.getHeight() < startY + height) {
					myEditor.getSelectedGates().add(gateView);
				}
			}
			myEditor.repaint();
		}
	}

	public void mouseMoved(MouseEvent e) {
		myMousePosition.x = e.getX();
		myMousePosition.y = e.getY();
	}
}
