package ru.amse.vorobiev.lce.view.impl;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.util.Map;

import ru.amse.vorobiev.lce.elements.IPlug;
import ru.amse.vorobiev.lce.ui.JEditor;
import ru.amse.vorobiev.lce.view.ICircuitView;
import ru.amse.vorobiev.lce.view.IPlugView;

public class PlugView implements IPlugView {
	private Point myCenter;
	private final IPlug myPlug;
	private final ICircuitView myCircuitView;
	private boolean isSelectedAsError;
	
	public PlugView(Point point, IPlug plug, ICircuitView circuitView) {
		myCenter = point;
		myPlug = plug;
		myCircuitView = circuitView;
		myCircuitView.addViewByElement(plug, this);
	}
	
	public Point getCenter() {
		return myCenter;
	}
	
	public IPlug getPlug() {
		return myPlug;
	}
	
	public void setCenter(Point point) {
		myCenter = point;
	}
	
	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		Map<IPlug, Boolean> map = myCircuitView.getCircuit().getInterpreter().getVisitedPlugs();
		if (map.get(myPlug) != null) {
			if (map.get(myPlug) == true) {
				g2.setColor(Color.GREEN);		
			} else {
				g2.setColor(Color.RED);
			}
		} else if(!isSelectedAsError) {
			g2.setColor(Color.BLACK);
		} else {
			g2.setColor(Color.RED);
		}
		
		g2.fillOval(myCenter.x - JEditor.PLUG_DIAMETER/2, myCenter.y - JEditor.PLUG_DIAMETER/2, 
				JEditor.PLUG_DIAMETER, JEditor.PLUG_DIAMETER);
	}

	public void setSelectedAsError(boolean condition) {
		isSelectedAsError = condition;
	}
}
