package ru.amse.vorobiev.lce.circuit;

import ru.amse.vorobiev.lce.elements.IGate;
import ru.amse.vorobiev.lce.elements.impl.AndGate;
import ru.amse.vorobiev.lce.elements.impl.InGate;
import ru.amse.vorobiev.lce.elements.impl.NotGate;
import ru.amse.vorobiev.lce.elements.impl.OrGate;
import ru.amse.vorobiev.lce.elements.impl.OutGate;
import ru.amse.vorobiev.lce.elements.impl.Wire;
import junit.framework.TestCase;

public class CircuitTest extends TestCase {
	private Circuit myCircuit;
	private IGate myIn;
	private IGate myOut;

	protected void setUp() throws Exception {
		myCircuit = new Circuit();
		
		IGate in = new InGate(6);
		IGate and1 = new AndGate();
		IGate and2 = new AndGate();
		IGate or1 = new OrGate();
		IGate or2 = new OrGate(3, 1);
		IGate not = new NotGate();
		IGate out = new OutGate(1);
		
		for (int i = 0; i < in.getOutputCount(); i++) {
			in.getOutput(i).setWire(new Wire());
		}
		
		in.getOutput(0).getWire().setTo(and1.getInput(0));
		in.getOutput(1).getWire().setTo(and1.getInput(1));
		in.getOutput(2).getWire().setTo(or1.getInput(0));
		in.getOutput(3).getWire().setTo(or1.getInput(1));
		in.getOutput(4).getWire().setTo(and2.getInput(1));
		in.getOutput(5).getWire().setTo(not.getInput(0));
		
		and1.getOutput(0).setWire(new Wire());
		or1.getOutput(0).setWire(new Wire());
		not.getOutput(0).setWire(new Wire());
		and2.getOutput(0).setWire(new Wire());
		or2.getOutput(0).setWire(new Wire());
		
		and1.getOutput(0).getWire().setTo(or2.getInput(0));
		or1.getOutput(0).getWire().setTo(and2.getInput(0));
		not.getOutput(0).getWire().setTo(or2.getInput(2));
		and2.getOutput(0).getWire().setTo(or2.getInput(1));
		or2.getOutput(0).getWire().setTo(out.getInput(0));
		
		myIn = in;
		myOut = out;
	}

	public void testAddGate() {
		AndGate gate = new AndGate();
		myCircuit.addGate(gate);
		assertTrue(myCircuit.getAllGates().contains(gate));
	}

	public void testAddSubcircuit() {
		myCircuit.addSubcircuit((InGate) myIn);
	}

	public void testGetInGates() {
		myCircuit.addSubcircuit((InGate) myIn);
		assertTrue(myCircuit.getInGates().contains(myIn) && myCircuit.getInGates().size() == 1);
	}
	
	public void testGetOutGates() {
		myCircuit.addSubcircuit((InGate) myIn);
		assertTrue(myCircuit.getOutGates().contains(myOut) && myCircuit.getOutGates().size() == 1);
	}

	public void testSize() {
		myCircuit.addSubcircuit((InGate) myIn);
		assertEquals(7, myCircuit.size());
	}

}
