package ru.amse.vorobiev.lce.interpreter;

import junit.framework.TestCase;
import ru.amse.vorobiev.lce.circuit.Circuit;
import ru.amse.vorobiev.lce.elements.*;
import ru.amse.vorobiev.lce.elements.impl.AndGate;
import ru.amse.vorobiev.lce.elements.impl.InGate;
import ru.amse.vorobiev.lce.elements.impl.NotGate;
import ru.amse.vorobiev.lce.elements.impl.OrGate;
import ru.amse.vorobiev.lce.elements.impl.OutGate;
import ru.amse.vorobiev.lce.elements.impl.Wire;
import ru.amse.vorobiev.lce.interpreter.Interpreter;

public class InterpreterTest extends TestCase {

	private Interpreter myInterpreter;

	protected void setUp() throws Exception {
		IGate in = new InGate(6);
		IGate and1 = new AndGate();
		IGate and2 = new AndGate();
		IGate or1 = new OrGate();
		IGate or2 = new OrGate(3, 1);
		IGate not = new NotGate();
		IGate out = new OutGate(1);
		
		for (int i = 0; i < in.getOutputCount(); i++) {
			in.getOutput(i).setWire(new Wire());
		}
		
		in.getOutput(0).getWire().setTo(and1.getInput(0));
		in.getOutput(1).getWire().setTo(and1.getInput(1));
		in.getOutput(2).getWire().setTo(or1.getInput(0));
		in.getOutput(3).getWire().setTo(or1.getInput(1));
		in.getOutput(4).getWire().setTo(and2.getInput(1));
		in.getOutput(5).getWire().setTo(not.getInput(0));
		
		and1.getOutput(0).setWire(new Wire());
		or1.getOutput(0).setWire(new Wire());
		not.getOutput(0).setWire(new Wire());
		and2.getOutput(0).setWire(new Wire());
		or2.getOutput(0).setWire(new Wire());
		
		and1.getOutput(0).getWire().setTo(or2.getInput(0));
		or1.getOutput(0).getWire().setTo(and2.getInput(0));
		not.getOutput(0).getWire().setTo(or2.getInput(2));
		and2.getOutput(0).getWire().setTo(or2.getInput(1));
		or2.getOutput(0).getWire().setTo(out.getInput(0));
		
		Circuit circuit = new Circuit();
		circuit.addSubcircuit((InGate) in);

		myInterpreter = new Interpreter(circuit);
	}

	public void testInterprete110011() {
		for (InGate in : myInterpreter.getInGates()) {
			myInterpreter.setInputs(in, new boolean[] {true, true, false, false, true, true});
			myInterpreter.interprete();
			for (OutGate out : myInterpreter.getOutGates()) {
				IPlug plug = out.getInput(0);
				boolean signal = myInterpreter.getVisitedPlugs().get(plug);
				assertEquals(true, signal);
			}
		}
	}
	
	public void testInterprete101010() {
		for (InGate in : myInterpreter.getInGates()) {
			myInterpreter.setInputs(in, new boolean[] {true, false, true, false, true, false});
			myInterpreter.interprete();
			for (OutGate out : myInterpreter.getOutGates()) {
				IPlug plug = out.getInput(0);
				boolean signal = myInterpreter.getVisitedPlugs().get(plug);
				assertEquals(true, signal);
			}
		}
	}
	
	public void testInterprete011101() {
		for (InGate in : myInterpreter.getInGates()) {
			myInterpreter.setInputs(in, new boolean[] {false, true, true, true, false, true});
			myInterpreter.interprete();
			for (OutGate out : myInterpreter.getOutGates()) {
				IPlug plug = out.getInput(0);
				boolean signal = myInterpreter.getVisitedPlugs().get(plug);
				assertEquals(false, signal);
			}
		}
	}
	
	public void testInterprete000000() {
		for (InGate in : myInterpreter.getInGates()) {
			myInterpreter.setInputs(in, new boolean[] {false, false, false, false, false, false});
			myInterpreter.interprete();
			for (OutGate out : myInterpreter.getOutGates()) {
				IPlug plug = out.getInput(0);
				boolean signal = myInterpreter.getVisitedPlugs().get(plug);
				assertEquals(true, signal);
			}
		}
	}
	
	public void testSetInputs() {
		boolean[] inputs = new boolean[] {true, true, false, false, true, true};
		InGate in = new InGate(6);
		myInterpreter.setInputs(in, inputs);
		boolean[] newInputs = new boolean[in.getOutputCount()];
		for (int i =0; i < in.getOutputCount(); i++) {
			newInputs[i] = myInterpreter.getVisitedPlugs().get(in.getOutput(i));
		}
		for (int i = 0; i < inputs.length; i++) {
			assertEquals(inputs[i], newInputs[i]);
		}
	}
	
}
