package ru.amse.vorobiev.lce.validator;

import java.util.List;
import ru.amse.vorobiev.lce.circuit.Circuit;
import ru.amse.vorobiev.lce.elements.IGate;
import ru.amse.vorobiev.lce.elements.impl.AndGate;
import ru.amse.vorobiev.lce.elements.impl.InGate;
import ru.amse.vorobiev.lce.elements.impl.NotGate;
import ru.amse.vorobiev.lce.elements.impl.OrGate;
import ru.amse.vorobiev.lce.elements.impl.OutGate;
import ru.amse.vorobiev.lce.elements.impl.Wire;
import junit.framework.TestCase;

public class ValidatorTest extends TestCase {
	private Validator myValidator;
	private Circuit myCircuit;

	protected void setUp() throws Exception {
		IGate in = new InGate(6);
		IGate and1 = new AndGate();
		IGate and2 = new AndGate();
		IGate or1 = new OrGate();
		IGate or2 = new OrGate(3, 1);
		IGate not = new NotGate();
		IGate out = new OutGate(1);
		
		for (int i = 1; i < in.getOutputCount(); i++) {
			in.getOutput(i).setWire(new Wire());
		}
		
//		in.getOutput(0).getWire().setTo(and1.getInput(0));
		in.getOutput(1).getWire().setTo(and1.getInput(1));
		in.getOutput(2).getWire().setTo(or1.getInput(0));
		in.getOutput(3).getWire().setTo(or1.getInput(1));
		in.getOutput(4).getWire().setTo(and2.getInput(1));
		in.getOutput(5).getWire().setTo(not.getInput(0));
		
		and1.getOutput(0).setWire(new Wire());
		or1.getOutput(0).setWire(new Wire());
		not.getOutput(0).setWire(new Wire());
		and2.getOutput(0).setWire(new Wire());
		or2.getOutput(0).setWire(new Wire());
		
		and1.getOutput(0).getWire().setTo(or2.getInput(0));
		or1.getOutput(0).getWire().setTo(and2.getInput(0));
		not.getOutput(0).getWire().setTo(or2.getInput(2));
		and2.getOutput(0).getWire().setTo(or2.getInput(1));
		or2.getOutput(0).getWire().setTo(out.getInput(0));
		
		Circuit circuit = new Circuit();
		circuit.addSubcircuit((InGate) in);
		
		myCircuit = circuit;
		myValidator = new Validator();
	}

	public void testValidateWithNoErrors() {
		IGate in = new InGate(6);
		IGate and1 = new AndGate();
		IGate and2 = new AndGate();
		IGate or1 = new OrGate();
		IGate or2 = new OrGate(3, 1);
		IGate not = new NotGate();
		IGate out = new OutGate(1);
		
		for (int i = 0; i < in.getOutputCount(); i++) {
			in.getOutput(i).setWire(new Wire());
		}
		
		in.getOutput(0).getWire().setTo(and1.getInput(0));
		in.getOutput(1).getWire().setTo(and1.getInput(1));
		in.getOutput(2).getWire().setTo(or1.getInput(0));
		in.getOutput(3).getWire().setTo(or1.getInput(1));
		in.getOutput(4).getWire().setTo(and2.getInput(1));
		in.getOutput(5).getWire().setTo(not.getInput(0));
		
		and1.getOutput(0).setWire(new Wire());
		or1.getOutput(0).setWire(new Wire());
		not.getOutput(0).setWire(new Wire());
		and2.getOutput(0).setWire(new Wire());
		or2.getOutput(0).setWire(new Wire());
		
		and1.getOutput(0).getWire().setTo(or2.getInput(0));
		or1.getOutput(0).getWire().setTo(and2.getInput(0));
		not.getOutput(0).getWire().setTo(or2.getInput(2));
		and2.getOutput(0).getWire().setTo(or2.getInput(1));
		or2.getOutput(0).getWire().setTo(out.getInput(0));
		
		Circuit circuit = new Circuit();
		circuit.addSubcircuit((InGate) in);

		myValidator = new Validator();
		
		List<IError> errors = myValidator.validate(circuit);
		assertEquals(true, errors.isEmpty());
	}
	

	public void testValidateWithEmptyPlugErrors() {
		List<IError> errors = myValidator.validate(myCircuit);
		assertEquals(2, errors.size());
	}
	
	public void testValidateWithHavingCyclesError() {
		IGate in = new InGate(6);
		IGate and1 = new AndGate();
		IGate and2 = new AndGate();
		IGate or1 = new OrGate(3, 1);
		IGate or2 = new OrGate();
		IGate not = new NotGate();
		IGate out = new OutGate(1);
		
		for (int i = 0; i < in.getOutputCount(); i++) {
			in.getOutput(i).setWire(new Wire());
		}
		
		in.getOutput(0).getWire().setTo(and1.getInput(0));
		in.getOutput(1).getWire().setTo(and1.getInput(1));
		in.getOutput(2).getWire().setTo(or1.getInput(0));
		in.getOutput(3).getWire().setTo(or1.getInput(1));
		in.getOutput(4).getWire().setTo(and2.getInput(1));
		in.getOutput(5).getWire().setTo(not.getInput(0));
		
		and1.getOutput(0).setWire(new Wire());
		or1.getOutput(0).setWire(new Wire());
		not.getOutput(0).setWire(new Wire());
		and2.getOutput(0).setWire(new Wire());
		or2.getOutput(0).setWire(new Wire());
		
		and1.getOutput(0).getWire().setTo(or2.getInput(0));
		or1.getOutput(0).getWire().setTo(and2.getInput(0));
		not.getOutput(0).getWire().setTo(or2.getInput(1));
		and2.getOutput(0).getWire().setTo(or1.getInput(2));
		or2.getOutput(0).getWire().setTo(out.getInput(0));
		
		Circuit circuit = new Circuit();
		circuit.addSubcircuit((InGate) in);

		myValidator = new Validator();
		
		List<IError> errors = myValidator.validate(circuit);
		assertTrue(errors.get(0) instanceof StructureError);
	}

}
